<?php
namespace Estoque\Repositories;

use \Geral\Util\DateUtil;
use Setforms2\Shared\Registry;

class Posestdia extends \Doctrine\ORM\EntityRepository
{

    CONST STATUS_S = 'S';

    /**
     * Verifica se há produto irregular no dia.
     *
     * @param $nrorg
     * @param $filial
     * @param $dtinicial
     * @param $dtfinal
     * @param $dtfinal
     *
     * @return bool|mixed
     */
    public function verificaProdutoIrregular($nrorg, $filial, $dtinicial, $dtfinal){
        $sql = 'SELECT P.*
                  FROM POSESTDIA P,
                       (SELECT CDFILIAL, CDPRODUTO, CDALMOXARIFE, CDLOCALESTOQ,
                               MAX(DTPOSIESTQ) AS DTPOSIESTQ, NRORG
                          FROM POSESTDIA
                         WHERE NRORG = :P_NRORG
                           AND CDFILIAL = :P_CDFILIAL
                           AND DTPOSIESTQ BETWEEN :P_INICIAL AND :P_FINAL
                         GROUP BY CDFILIAL, CDPRODUTO, CDALMOXARIFE, CDLOCALESTOQ, NRORG) A
                 WHERE P.NRORG = :P_NRORG
                   AND P.CDFILIAL = :P_CDFILIAL
                   AND ((ROUND(P.QTESTOQDIA,5) < -0.00001) OR (ROUND(P.VRESTOQDIA,5) < -0.00001))
                   AND P.NRORG = A.NRORG
                   AND P.CDFILIAL = A.CDFILIAL
                   AND P.CDPRODUTO = A.CDPRODUTO
                   AND P.CDALMOXARIFE = A.CDALMOXARIFE
                   AND P.CDLOCALESTOQ = A.CDLOCALESTOQ
                   AND P.DTPOSIESTQ = A.DTPOSIESTQ';

        $params = array(
            'P_NRORG'    => $nrorg,
            'P_CDFILIAL' => $filial,
            'P_INICIAL'  => $dtinicial,
            'P_FINAL'    => $dtfinal,
        );

        $entityManager = $this->getEntityManager();
        $resultSetMapping = new \Doctrine\ORM\Query\ResultSetMappingBuilder($entityManager);
        $resultSetMapping->addRootEntityFromClassMetadata('Estoque\Entities\Posestdia', "P");

        $result = $entityManager->createNativeQuery($sql, $resultSetMapping)->execute($params);
        return count($result) > 0 ? $result : false;
    }

    /**
     * Verifica se há produto irregular no dia (versão para Fechamento Dia)
     *
     * @param $filial
     * @param $dtfinal
     *
     * @internal param $nrorg
     * @internal param $dtinicial
     * @return bool|mixed
     */
    public function verificaProdutoIrregularII($filial, $dtfinal){
        $sql = 'SELECT CDPRODUTO, P.VRESTOQDIA,P.QTESTOQDIA
						FROM POSESTDIA P
					WHERE (CDFILIAL = :P_CDFILIAL)
						AND ((P.QTESTOQDIA < -0.00001) OR (P.VRESTOQDIA < -0.00001))
						AND (DTPOSIESTQ =
    							(SELECT MAX(DTPOSIESTQ)
                    				FROM POSESTDIA A
                    			WHERE A.CDFILIAL   = P.CDFILIAL
                    				AND A.CDPRODUTO    = P.CDPRODUTO
                    				AND A.CDALMOXARIFE = P.CDALMOXARIFE
                    				AND A.CDLOCALESTOQ = P.CDLOCALESTOQ
                    				AND DTPOSIESTQ    <= :P_DTFINAL))';

        $params = array(
            'P_CDFILIAL' => $filial,
            'P_DTFINAL'    => $dtfinal,
        );

        $entityManager = $this->getEntityManager();
        $resultSetMapping = new \Doctrine\ORM\Query\ResultSetMappingBuilder($entityManager);
        $resultSetMapping->addRootEntityFromClassMetadata('Estoque\Entities\Posestdia', "P");

        $result = $entityManager->createNativeQuery($sql, $resultSetMapping)->execute($params);
        return count($result) > 0 ? $result : false;
    }



    /**
     * Verifica produtos a serem zerados pela rotina de ajuste/implantacao de estoque
     *
     * @param $cdfilial
     * @param $cdfilialitlancto
     * @param $dt_lancamento
     * @param $nr_lancamento
     * @param $nrorg
     *
     * @return bool|mixed
     */
    public function zeraEstoqueProdutos($cdfilial, $cdfilialitlancto, $dt_lancamento, $nr_lancamento, $nrorg){
        $sql = "SELECT
                   B.CDARVPROD, P.CDPRODUTO,  P.CDALMOXARIFE, P.CDLOCALESTOQ,
                   P.NRLOTEESTQ, P.NRSUBLOTE, B.NMPRODUTO, B.SGUNIDADE, P.QTESTOQDIA
                FROM
                   PRODUTO B, POSESTDIA P
                WHERE (P.CDFILIAL     = :P_CDFILIAL)
                  AND (P.CDPRODUTO    = B.CDPRODUTO)
                  AND (P.NRORG        = B.NRORG)
                  AND (P.QTESTOQDIA <> 0)
                  AND (P.NRORG        = :P_NRORG)
                  AND (P.DTPOSIESTQ = (SELECT MAX(DTPOSIESTQ)
                                         FROM POSESTDIA C
                                        WHERE (C.CDFILIAL     = P.CDFILIAL)
                                          AND (C.CDALMOXARIFE = P.CDALMOXARIFE)
                                          AND (C.CDLOCALESTOQ = P.CDLOCALESTOQ)
                                          AND (C.CDPRODUTO    = P.CDPRODUTO)
                                          AND (C.NRLOTEESTQ   = P.NRLOTEESTQ)
                                          AND (C.NRSUBLOTE    = P.NRSUBLOTE)
                                          AND (C.NRORG        = P.NRORG)
                                          AND (C.DTPOSIESTQ   <= to_date(:P_DTIMPLANT, 'DD/MM/YYYY HH24:mi:ss'))))
                  AND (NOT EXISTS(SELECT 1
                                    FROM ITLANCTOEST I
                                   WHERE (I.CDFILIAL     = :P_FILILANCTO )
                                     AND (I.NRLANCESTQ   = :P_NRLANCESTQ )
                                     AND (I.NRORG        = P.NRORG)
                                     AND (P.CDPRODUTO    = I.CDPRODMOVI  )
                                     AND (I.CDALMOXARIFE = P.CDALMOXARIFE)
                                     AND (I.CDLOCALESTOQ = P.CDLOCALESTOQ)
                                     AND (I.NRLOTEESTQ   = P.NRLOTEESTQ  )
                                     AND (I.NRSUBLOTE    = P.NRSUBLOTE   )))
                ORDER BY B.CDARVPROD, P.CDALMOXARIFE, P.CDLOCALESTOQ, P.NRLOTEESTQ, P.NRSUBLOTE";

        $params = array(
            "P_CDFILIAL"    => $cdfilial,
            "P_NRORG"       => $nrorg,
            "P_DTIMPLANT"   => $dt_lancamento,
            "P_FILILANCTO" => $cdfilialitlancto,
            "P_NRLANCESTQ"  => $nr_lancamento,
//            "P_DTIMPLANT2"  => '29/12/2012',
        );

        $entityManager = $this->getEntityManager();
        $result = $entityManager->getConnection()->fetchAll($sql, $params);
        return count($result)>0 ? $result : array();
    }


    public function verificaPosicaoEstoque(
        $filial, $nrorg, $cdProduto, $lote, $local, $subLote, $almox, $custo, \Setforms2\Shared\ORM\DateTime $dtPosicao, $qtd
    ){
        $dtPosicao = $dtPosicao->format(DateUtil::FORMATO_BRASILEIRO_DATAHORA);
        $dtPosEstDia = self::verificaDtPosicaoEstoque($filial, $nrorg, $cdProduto, $lote, $local, $subLote, $almox, $qtd, $dtPosicao);
        $dtAtual = DateUtil::getDataAtual()->format(DateUtil::FORMATO_BRASILEIRO_DATAHORA);

        $sql = "SELECT NVL(CUSTO_PRODUTO_LCEST(:P_CDFILIAL, D.CDPRODUTO, :P_INTPCUSTO,
                                              to_date(:P_DTPOSICAO, 'DD/MM/YYYY HH24:mi:ss'),
                                              to_date(:P_DATEATUAL, 'DD/MM/YYYY HH24:mi:ss'),
                                              D.CDALMOXARIFE, D.CDLOCALESTOQ, D.NRLOTEESTQ, D.NRSUBLOTE),
                          0) * P.VRFATOCONV AS VRCUSTOPROD,
                     (D.QTESTOQDIA/P.VRFATOCONV) AS QTESTOQDIA,
                     P.SGUNIDADE, DTVALILOTE, P.NMPRODUTO,  'S' IDCNTRESTOQ, D.CDALMOXARIFE,
                     D.CDLOCALESTOQ, D.NRLOTEESTQ, D.NRSUBLOTE, P.VRFATOCONV,
                     (QTESTOQANT/P.VRFATOCONV)  AS QTESTOQANT, D.CDPRODUTO, CDPRODESTO
                FROM PRODUTO P, POSESTDIA D, LOTE L
                WHERE D.CDFILIAL     = :P_CDFILIAL
                  AND D.NRORG        = :P_NRORG
                  AND D.CDALMOXARIFE = :P_CDALMOXARIFE
                  AND D.CDLOCALESTOQ = :P_CDLOCALESTOQ
                  AND D.NRLOTEESTQ   = :P_NRLOTEESTQ
                  AND D.NRSUBLOTE    = :P_NRSUBLOTE
                  AND D.DTPOSIESTQ   = to_Date(:P_DTPOSIESTQ, 'DD/MM/YYYY HH24:mi:ss')
                  AND P.NRORG        = D.NRORG
                  AND P.CDPRODESTO   = D.CDPRODUTO
                  AND P.CDPRODUTO    = :P_CDPRODUTO
                  AND L.NRORG        = P.NRORG
                  AND L.CDFILIAL     = D.CDFILIAL
                  AND L.NRLOTEESTQ   = D.NRLOTEESTQ
                  AND L.NRSUBLOTE    = D.NRSUBLOTE
                  AND L.CDPRODUTO    = D.CDPRODUTO";

        $params = array(
            'P_CDFILIAL' => $filial,
            'P_INTPCUSTO' => $custo,
            'P_DTPOSICAO' => $dtPosicao,
            'P_DATEATUAL' => $dtAtual,
            'P_NRORG' => $nrorg,
            'P_CDALMOXARIFE' => $almox,
            'P_CDLOCALESTOQ' => $local,
            'P_NRLOTEESTQ' => $lote,
            'P_NRSUBLOTE' => $subLote,
            'P_DTPOSIESTQ' => $dtPosEstDia,
            'P_CDPRODUTO' => $cdProduto,
        );

        $result = $this->getEntityManager()->getConnection()->fetchAll($sql, $params);
        return ($result) ? current($result) : false;
    }

    public function retornaDataPosicaoEstoque($filial, $cdalmoxarife, $cdlocalestoq, $nrloteestq, $nrsublote, $cdprodesto, $dtsaida){
        $sql = <<<SQL
        SELECT MAX(DTPOSIESTQ) DTPOSIESTQ
            FROM POSESTDIA
            WHERE (CDFILIAL = :P_CDFILIAL)
              AND (CDALMOXARIFE = :P_CDALMOXARIFE)
              AND (CDLOCALESTOQ = :P_CDLOCALESTOQ)
              AND (NRLOTEESTQ = :P_NRLOTEESTQ)
              AND (NRSUBLOTE = :P_NRSUBLOTE)
              AND (CDPRODUTO = :P_CDPRODESTO)
              AND (DTPOSIESTQ <= :P_DTSAIDA)
SQL;

        $params = array(
            "P_CDFILIAL" => $filial,
            "P_CDALMOXARIFE" => $cdalmoxarife,
            "P_CDLOCALESTOQ" => $cdlocalestoq,
            "P_NRLOTEESTQ" => $nrloteestq,
            "P_NRSUBLOTE" => $nrsublote,
            "P_CDPRODESTO" => $cdprodesto,
            "P_DTSAIDA" => $dtsaida
        );

        try{
            return self::getEntityManager()->getConnection()->fetchColumn($sql, $params);
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function retornaCustoProdutoEstoque($filial, $intpcusto, $dtposicao, $cdalmoxarife, $cdlocalestoq, $cdproduto, $nrloteestq, $nrsublote, $dtposiestq, $qtde){
        $sql = <<<SQL
        SELECT NVL(CUSTO_PRODUTO_LCEST(:P_CDFILIAL, D.CDPRODUTO, :P_INTPCUSTO, :P_DTPOSICAO, SYSDATE,
                   D.CDALMOXARIFE, D.CDLOCALESTOQ, D.NRLOTEESTQ, D.NRSUBLOTE),0) * P.VRFATOCONV AS VRCUSTOPROD,
                   (D.QTESTOQDIA/P.VRFATOCONV) AS QTESTOQDIA, P.SGUNIDADE,
                   DTVALILOTE, P.NMPRODUTO,  'S' IDCNTRESTOQ, D.CDALMOXARIFE,
                   D.CDLOCALESTOQ, D.NRLOTEESTQ, D.NRSUBLOTE, P.VRFATOCONV,
                   (QTESTOQANT/P.VRFATOCONV)  AS QTESTOQANT, D.CDPRODUTO, CDPRODESTO
            FROM PRODUTO P, POSESTDIA D, LOTE L
            WHERE (D.CDFILIAL = :P_CDFILIAL)
              AND (D.CDALMOXARIFE = :P_CDALMOXARIFE)
              AND (D.CDLOCALESTOQ = :P_CDLOCALESTOQ)
              AND (P.CDPRODUTO = :P_CDPRODUTO)
              AND (D.NRLOTEESTQ = :P_NRLOTEESTQ)
              AND (D.NRSUBLOTE = :P_NRSUBLOTE)
              AND (DTPOSIESTQ = :P_DTPOSIESTQ)
              AND (D.CDPRODUTO = P.CDPRODESTO)
              AND (D.QTESTOQDIA > :P_QTDE OR :P_QTDE = -1)
              AND (D.CDFILIAL = L.CDFILIAL)
              AND (D.NRLOTEESTQ = L.NRLOTEESTQ)
              AND (D.NRSUBLOTE = L.NRSUBLOTE)
              AND (D.CDPRODUTO = L.CDPRODUTO)
SQL;

        $params = array(
            "P_CDFILIAL" => $filial,
            "P_INTPCUSTO" => $intpcusto,
            "P_DTPOSICAO" => $dtposicao,
            "P_CDALMOXARIFE" => $cdalmoxarife,
            "P_CDLOCALESTOQ" => $cdlocalestoq,
            "P_CDPRODUTO" => $cdproduto,
            "P_NRLOTEESTQ" => $nrloteestq,
            "P_NRSUBLOTE" => $nrsublote,
            "P_DTPOSIESTQ" => $dtposiestq,
            "P_QTDE" => $qtde
        );

        try{
            return self::getEntityManager()->getConnection()->fetchAll($sql, $params);
        } catch(\Exception $e) {
            throw $e;
        }
    }

    private function verificaDtPosicaoEstoque(
        $filial, $nrorg, $cdProduto, $lote, $local, $subLote, $almox, $qtd, $dtPosicao
    ){
        $sql = "SELECT MAX(DTPOSIESTQ) DTPOSIESTQ
                  FROM POSESTDIA
                 WHERE CDFILIAL = :P_CDFILIAL
                   AND NRORG = :P_NRORG
                   AND CDALMOXARIFE = :P_CDALMOXARIFE
                   AND CDLOCALESTOQ = :P_CDLOCALESTOQ
                   AND NRLOTEESTQ = :P_NRLOTEESTQ
                   AND NRSUBLOTE = :P_NRSUBLOTE
                   AND CDPRODUTO = :P_CDPRODUTO
                   AND DTPOSIESTQ <= :P_DTPOSICAO
                   AND (QTESTOQDIA > :P_QTDE OR :P_QTDE = -1)";

        $params = array(
            'P_CDFILIAL' => $filial,
            'P_NRORG' => $nrorg,
            'P_CDALMOXARIFE' => $almox,
            'P_CDLOCALESTOQ' => $local,
            'P_NRLOTEESTQ' => $lote,
            'P_NRSUBLOTE' => $subLote,
            'P_CDPRODUTO' => $cdProduto,
            'P_DTPOSICAO' => $dtPosicao,
            'P_QTDE' => $qtd
        );

        $result = $this->getEntityManager()->getConnection()->fetchColumn($sql, $params);
        return ($result) ? $result : false;
    }


    public function verificaQtdProdutoDisponivel(
        $nrorg, $filial, $almox, $local, $lote, $subLote, $filialLanc,
        $nrLancto, $seqItem, $produtoMovi, $produto, $dtLancto, $qtTotal, $custo
    ){

        if (empty($custo) === true) {
            $custo = 3;
        }

        $sql = "SELECT P.CDARVPROD, P.CDPRODUTO,  P.NMPRODUTO,  P.SGUNIDADE, D. VRESTOQDIA,
                       P.VRFATOCONV, TRUNC((D.QTESTOQDIA+NVL(C.QTLANCTOEST,0)/P.VRFATOCONV), 5) AS QTESTOQDIA,
                       TO_CHAR((D.QTESTOQDIA+NVL(C.QTLANCTOEST,0)/P.VRFATOCONV) - :P_QTTOTLANCTO) AS QTNEG,
                       CUSTO_PRODUTO_LCEST(:P_CDFILIAL, :P_CDPRODUTO,
                                                     TO_NUMBER(:P_CDTIPCUSTEST), TO_DATE(:P_DTPOSICAO), SYSDATE,
                                                     :P_CDALMOXARIFE,:P_CDLOCALESTOQ, :P_NRLOTEESTQ, :P_NRSUBLOTE) VRCUSTPROD
                  FROM PRODUTO P, POSESTDIA D
                  LEFT JOIN
                       (SELECT CDPRODUTO, CDPRODMOVI, SUM(QTLANCTOEST) AS QTLANCTOEST
                          FROM ITLANCTOEST
                          WHERE (CDFILIAL   = :P_CDFILILANC) AND
                                (NRLANCESTQ = :P_NRLANCESTQ) AND
                                (NRSEQUITEM = :P_NRSEQUITEM) AND
                                (CDPRODUTO  = :P_CDPRODUTO)  AND
                                (NVL(IDEVOLESTOQ, :P_IDEVOLESTOQ) = :P_IDEVOLESTOQ)
                          GROUP BY CDPRODUTO, CDPRODMOVI) C ON (:P_CDPRODMOVI    = C.CDPRODMOVI)
                 WHERE (P.CDPRODUTO    = :P_CDPRODMOVI)   AND
                       (D.CDFILIAL     = :P_CDFILIAL)     AND
                       (D.CDALMOXARIFE = :P_CDALMOXARIFE) AND
                       (D.CDLOCALESTOQ = :P_CDLOCALESTOQ) AND
                       (D.NRLOTEESTQ   = :P_NRLOTEESTQ)   AND
                       (D.NRSUBLOTE    = :P_NRSUBLOTE)    AND
                       (P.CDPRODESTO   = D.CDPRODUTO)     AND
                       ((D.DTPOSIESTQ  = (SELECT MAX(DTPOSIESTQ)
                                            FROM POSESTDIA
                                           WHERE (CDFILIAL     = D.CDFILIAL)     AND
                                                 (CDALMOXARIFE = D.CDALMOXARIFE) AND
                                                 (CDLOCALESTOQ = D.CDLOCALESTOQ) AND
                                                 (NRLOTEESTQ   = D.NRLOTEESTQ)   AND
                                                 (NRSUBLOTE    = D.NRSUBLOTE)    AND
                                                 (CDPRODUTO    = P.CDPRODESTO)    AND
                                                 (DTPOSIESTQ  <= TO_DATE(:P_DTPOSICAO) +1 ))) OR (DTPOSIESTQ IS NULL))
                  ORDER BY 1";

        $params = array(
            'P_CDFILIAL' => $filial,
            'P_CDALMOXARIFE' => $almox,
            'P_CDLOCALESTOQ' => $local,
            'P_NRLOTEESTQ' => $lote,
            'P_NRSUBLOTE' => $subLote,
            'P_CDFILILANC' => $filialLanc,
            'P_NRLANCESTQ' => $nrLancto,
            'P_NRSEQUITEM' => $seqItem,
            'P_IDEVOLESTOQ' => self::STATUS_S,
            'P_CDPRODMOVI' => $produtoMovi,
            'P_CDPRODUTO' => $produto,
            'P_DTPOSICAO' => $dtLancto,
            'P_QTTOTLANCTO' => $qtTotal,
            'P_CDTIPCUSTEST' => $custo
        );

        $result = $this->getEntityManager()->getConnection()->fetchAll($sql, $params);
        return count($result)>0 ? current($result) : false;
    }

    public function deletaPosicaoEstoque($filial, $almoxarifado, $local, $produto_ini, $produto_fin, $data){
        if(!$almoxarifado && !$local){
            $sql = <<<SQL
                            DELETE
                            FROM POSESTDIA P
                            WHERE
                               (EXISTS (SELECT PR.CDPRODUTO
                                        FROM PRODUTO PR
                                        WHERE PR.CDARVPROD BETWEEN :P_CDPRODINI AND :P_CDPRODFIM  AND
                                              PR.CDPRODUTO = P.CDPRODUTO))  AND
                               (P.DTPOSIESTQ   >=:P_DTPOSIESTQ)            AND
                               (P.CDFILIAL     = :P_CDFILIAL)
SQL;

            $params = array(
                "P_CDPRODINI" => $produto_ini,
                "P_CDPRODFIM" => $produto_fin,
                "P_DTPOSIESTQ" => $data,
                "P_CDFILIAL" => $filial,
            );
        }
        elseif($almoxarifado && !$local){
            $sql = <<<SQL
                            DELETE
                            FROM POSESTDIA P
                            WHERE
                               (EXISTS (SELECT PR.CDPRODUTO
                                        FROM PRODUTO PR
                                        WHERE PR.CDARVPROD BETWEEN :P_CDPRODINI AND :P_CDPRODFIM  AND
                                              PR.CDPRODUTO = P.CDPRODUTO))  AND
                               (P.DTPOSIESTQ   >=:P_DTPOSIESTQ)            AND
                               (P.CDALMOXARIFE = :P_CDALMOXARIFE ) AND
                               (P.CDFILIAL     = :P_CDFILIAL)
SQL;

            $params = array(
                "P_CDPRODINI" => $produto_ini,
                "P_CDPRODFIM" => $produto_fin,
                "P_DTPOSIESTQ" => $data,
                "P_CDFILIAL" => $filial,
                "P_CDALMOXARIFE" => $almoxarifado,
            );
        }elseif($almoxarifado && $local){
            $sql = <<<SQL
                            DELETE
                            FROM POSESTDIA P
                            WHERE
                               (EXISTS (SELECT PR.CDPRODUTO
                                        FROM PRODUTO PR
                                        WHERE PR.CDARVPROD BETWEEN :P_CDPRODINI AND :P_CDPRODFIM  AND
                                              PR.CDPRODUTO = P.CDPRODUTO))  AND
                               (P.DTPOSIESTQ   >= :P_DTPOSIESTQ)            AND
                               (P.CDALMOXARIFE = :P_CDALMOXARIFE ) AND
                               (P.CDLOCALESTOQ = :P_CDLOCALESTOQ ) AND
                               (P.CDFILIAL     = :P_CDFILIAL)
SQL;

            $params = array(
                "P_CDPRODINI" => $produto_ini,
                "P_CDPRODFIM" => $produto_fin,
                "P_DTPOSIESTQ" => $data,
                "P_CDFILIAL" => $filial,
                "P_CDALMOXARIFE" => $almoxarifado,
                "P_CDLOCALESTOQ" => $local,
            );
        }elseif(!$almoxarifado){
            $sql = <<<SQL
                            DELETE
                            FROM POSESTCONS P
                            WHERE
                               (P.CDFILIAL    = :P_CDFILIAL)    AND
                               (P.DTPOSESTCON >= :P_DTPOSESTCON)  AND
                               (EXISTS (SELECT PR.CDPRODUTO
                                        FROM PRODUTO PR
                                        WHERE PR.CDPRODUTO = P.CDPRODUTO AND
                                              PR.CDARVPROD BETWEEN :P_CDPRODINI AND :P_CDPRODFIM))
SQL;

            $params = array(
                "P_CDPRODINI" => $produto_ini,
                "P_CDPRODFIM" => $produto_fin,
                "P_DTPOSESTCON" => $data,
                "P_CDFILIAL" => $filial,
            );
        }

        try{
            return self::getEntityManager()->getConnection()->executeQuery($sql, $params);
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function verificaEstoque($filial, $prodini, $prodfin)
    {
        $sql = <<<SQL
          SELECT B.CDPRODUTO, A.CDPRODESTO, A.NMPRODUTO, MAX(B.QTESTOQDIA) AS QTESTOQDIA
          FROM POSESTDIA B,
          (SELECT CDPRODUTO, NMPRODUTO, SGUNIDADE, CDARVPROD, CDPRODESTO
          FROM PRODUTO
          WHERE CDARVPROD BETWEEN :P_PRODINI AND :P_PRODFIM) A,
          (SELECT CDFILIAL, CDPRODUTO, NRLOTEESTQ, NRSUBLOTE, MAX(DTPOSIESTQ) AS DTPOSIESTQ
          FROM POSESTDIA
          GROUP BY CDFILIAL, CDPRODUTO, NRLOTEESTQ, NRSUBLOTE) D
          WHERE B.CDFILIAL = :P_FILIAL
            AND ABS(B.QTESTOQDIA) > 0.00001
            AND B.CDPRODUTO = A.CDPRODUTO
            AND A.CDPRODUTO = A.CDPRODESTO
            AND B.CDFILIAL = D.CDFILIAL
            AND B.CDPRODUTO = D.CDPRODUTO
            AND B.NRLOTEESTQ = D.NRLOTEESTQ
            AND B.NRSUBLOTE = D.NRSUBLOTE
            AND B.DTPOSIESTQ = D.DTPOSIESTQ
          GROUP BY B.CDPRODUTO, A.NMPRODUTO, A.SGUNIDADE, A.CDPRODESTO, A.CDARVPROD
          ORDER BY A.NMPRODUTO
SQL;
        $params = array(
            "P_FILIAL" => $filial,
            "P_PRODINI" => $prodini,
            "P_PRODFIM" => $prodfin,
        );

        try{
            return self::getEntityManager()->getConnection()->fetchAll($sql, $params);
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function deletaPosicaoEstoquePorProduto($filial, $produto){

        $sql = <<<SQL
            DELETE FROM POSESTDIA
            WHERE (CDFILIAL  = :P_CDFILIAL)  AND
                  (CDPRODUTO = :P_CDPRODUTO)
SQL;

        $params = array(
            "P_CDFILIAL" => $filial,
            "P_CDPRODUTO" => $produto,
        );

        try{
            return self::getEntityManager()->getConnection()->executeQuery($sql, $params);
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function consultaCustoQuantidadeProduto(
        $filial, $tipoCusto, $produto, $almoxarifado, $localEstoque, $lote, $subLote, $dtLancamento
    )
    {
        $sql = <<<SQL
        SELECT NVL(CUSTO_PRODUTO_LCEST(:P_CDFILIAL, D.CDPRODUTO, :P_INTPCUSTO, :P_DTLANC, SYSDATE,
               D.CDALMOXARIFE, D.CDLOCALESTOQ, D.NRLOTEESTQ, D.NRSUBLOTE),0) * P.VRFATOCONV AS VRCUSTOPROD,
               (D.QTESTOQDIA/P.VRFATOCONV) AS QTESTOQDIA
          FROM PRODUTO P, POSESTDIA D, LOTE L
         WHERE P.CDPRODUTO = :P_CDPRODUTO
           AND D.CDFILIAL = :P_CDFILIAL
           AND D.CDALMOXARIFE = :P_CDALMOXARIFE
           AND D.CDLOCALESTOQ = :P_CDLOCALESTOQ
           AND D.NRLOTEESTQ = :P_NRLOTEESTQ
           AND D.NRSUBLOTE = :P_NRSUBLOTE
           AND D.CDPRODUTO = P.CDPRODESTO
           AND (D.QTESTOQDIA > 0)
           AND D.CDFILIAL = L.CDFILIAL
           AND D.NRLOTEESTQ = L.NRLOTEESTQ
           AND D.NRSUBLOTE = L.NRSUBLOTE
           AND D.CDPRODUTO = L.CDPRODUTO
           AND D.DTPOSIESTQ = (SELECT MAX(B.DTPOSIESTQ)
                                 FROM POSESTDIA B
                                WHERE B.CDFILIAL     = D.CDFILIAL
                                  AND B.CDPRODUTO    = D.CDPRODUTO
                                  AND B.CDALMOXARIFE = D.CDALMOXARIFE
                                  AND B.CDLOCALESTOQ = D.CDLOCALESTOQ
                                  AND B.NRLOTEESTQ   = D.NRLOTEESTQ
                                  AND B.NRSUBLOTE    = D.NRSUBLOTE
                                  AND B.DTPOSIESTQ   <= TO_DATE(:P_DTLANC) + 1)
SQL;

        $params = array(
            "P_CDFILIAL" => $filial,
            "P_INTPCUSTO" => $tipoCusto,
            "P_CDALMOXARIFE" => $almoxarifado,
            "P_CDLOCALESTOQ" => $localEstoque,
            "P_CDPRODUTO" => $produto,
            "P_NRLOTEESTQ" => $lote,
            "P_NRSUBLOTE" => $subLote,
            "P_DTLANC" => $dtLancamento
        );

        try{
            return $this->getEntityManager()->getConnection()->fetchAssoc($sql, $params);
        } catch(\Exception $e) {
            throw $e;
        }
    }
}