create or replace
PACKAGE INT_SAP_TEK_TI AS
  /* Criacao de um tipo que armazenara as informacoes da tabela EXPORTACAOTI */
  TYPE TAB_SAP_TI IS RECORD(CDEMPRESATI  VARCHAR2(2),
                            CDFILIALTI   VARCHAR2(4),
                            NRNUMERORETI VARCHAR2(4),
                            DTCONTABILTI VARCHAR2(8),
                            CDFILIAL2TI  VARCHAR2(4),
                            INSCRICAOTI  VARCHAR2(14),
                            PRODUTOTI    VARCHAR2(20),
                            QUANTIDADETI FLOAT,
                            VRUNITARIOTI FLOAT,
                            VRESTOQTI    FLOAT);

  -- Tipo que sera retornado pela funcao GET_TI
  TYPE TYPE_TI IS TABLE OF TAB_SAP_TI;
  
  -- Converte o produto nos casos em que a filial seja OL
  FUNCTION GETPRODUTO(P_CDPRODUTOTI VARCHAR2, P_IDFUNCAOFILI VARCHAR2) RETURN VARCHAR2;
  
  -- Pega o fator de conversao dos produtos do OL para que seja, posteriormente, 
  -- multiplicado a quantidade do produto, o valor "unitario" nao sera alterado.
  FUNCTION GETFATORCONV(P_CDARVPROD VARCHAR2, P_IDFUNCAOFILI VARCHAR2) RETURN FLOAT;

  -- Retorna os valores que podem ser exportados, desconsiderando dados j exportados.
  FUNCTION GET_TI(P_FILIAL VARCHAR2, P_DATAINI VARCHAR2, P_DATAFINAL VARCHAR2, P_IDFUNCAOFILI VARCHAR2) RETURN TYPE_TI PIPELINED;

  -- Exporta os dados para tabela intermediaria do SAP, "USR_SAP_EXPORTACAOTI"
  PROCEDURE EXPORTA_TI_SAP(P_FILIAL VARCHAR2, P_DATAINI DATE, P_DATAFINAL DATE, P_TCDFILIAL VARCHAR2);

END INT_SAP_TEK_TI;
/

create or replace
PACKAGE BODY INT_SAP_TEK_TI AS
   /* Funcao para produtos */
   FUNCTION GETPRODUTO(P_CDPRODUTOTI VARCHAR2, P_IDFUNCAOFILI VARCHAR2) RETURN VARCHAR2
   IS
      V_CDPRODUTO    VARCHAR2(20);
      V_CDPRODOL     VARCHAR2(10);
      V_CDPRODESTO   VARCHAR2(10);
      V_CDARVPRODORI VARCHAR2(20);
   BEGIN
      IF P_IDFUNCAOFILI = 'O' THEN
         /* Faz a conversao dos produtos que pertencerem a filial OL (relacionamento produto OL)*/
         BEGIN
            SELECT CDPRODUTO
              INTO V_CDPRODUTO
              FROM PRODUTO
             WHERE CDPRODINTE = TRIM(P_CDPRODUTOTI);
         EXCEPTION
            WHEN OTHERS THEN
               SELECT CDPRODUTO
                 INTO V_CDPRODUTO
                 FROM PRODUTO
                WHERE CDARVPROD = TRIM(P_CDPRODUTOTI);
         END;

         BEGIN
            SELECT MIN(CDPRARVORIG) CDPRARVORIG
              INTO V_CDPRODOL
              FROM RELACPRODOL
             WHERE CDPRODARVOL = V_CDPRODUTO;
         EXCEPTION
            WHEN OTHERS THEN
               V_CDPRODOL := V_CDPRODUTO;
         END;

         IF V_CDPRODOL IS NULL THEN
            V_CDPRODOL := V_CDPRODUTO;
         END IF;

         BEGIN
            SELECT CDPRODESTO
              INTO V_CDPRODESTO
              FROM PRODUTO
             WHERE CDPRODUTO = V_CDPRODOL;
         EXCEPTION
            WHEN OTHERS THEN
               V_CDPRODESTO := V_CDPRODOL;
         END;

         SELECT CDARVPROD
           INTO V_CDARVPRODORI
           FROM PRODUTO
          WHERE CDPRODUTO = V_CDPRODESTO;
      ELSE
         /* Retorna o proprio produto caso a filial nao seja OL */
         V_CDARVPRODORI := P_CDPRODUTOTI;
      END IF;

      RETURN V_CDARVPRODORI;

   END GETPRODUTO;

   /* Funcao para o fator de conversao */
   FUNCTION GETFATORCONV(P_CDARVPROD VARCHAR2, P_IDFUNCAOFILI VARCHAR2) RETURN FLOAT
   IS
      V_VRFATOCONV FLOAT;
      V_PRODUTOTI  VARCHAR2(20);
   BEGIN
      IF P_IDFUNCAOFILI = 'O' THEN
         /* Retorna o fator de conversao do produto (relacionado ao OL) que pertencer a filial OL */
         V_PRODUTOTI := GETPRODUTO(P_CDARVPROD, P_IDFUNCAOFILI);

         BEGIN
            SELECT NVL(VRFATOCONV, 1)
              INTO V_VRFATOCONV
              FROM PRODUTO
             WHERE CDARVPROD = V_PRODUTOTI;
         EXCEPTION
            WHEN OTHERS THEN
               V_VRFATOCONV := 1;
         END;
      ELSE
         /* Caso a filial no seja OL, o valor do fator de conversao eh 1 */
         V_VRFATOCONV := 1;
      END IF;

      RETURN V_VRFATOCONV;
   END GETFATORCONV;

   /* Funcao que retorna os dados a serem exportados levando em consideracao se ja foram ou nao exportados */
   FUNCTION GET_TI(P_FILIAL VARCHAR2, P_DATAINI VARCHAR2, P_DATAFINAL VARCHAR2, P_IDFUNCAOFILI VARCHAR2) RETURN TYPE_TI PIPELINED
   IS
      /* Tipo de dado que sera retornado pela funcao */
      TI_SAP TAB_SAP_TI;

      V_CDEMPRESATI  VARCHAR2(2);
      V_CDFILIALTI   VARCHAR2(4);
      V_NRNUMERORETI VARCHAR2(4);
      V_DTCONTABILTI VARCHAR2(8);
      V_CDFILIAL2TI  VARCHAR2(4);
      V_INSCRICAOTI  VARCHAR2(14);
      V_PRODUTOTI    VARCHAR2(20);
      V_QUANTIDADETI FLOAT;
      V_VRUNITARIOTI FLOAT;
      V_VRESTOQTI    FLOAT;
      CURSOR C_EXPORTATI IS (SELECT E.CDEMPRESATI, E.CDFILIALTI, E.NRNUMERORETI, E.DTCONTABILTI, E.CDFILIAL2TI,
                                    E.INSCRICAOTI, E.PRODUTOTI, E.QUANTIDADETI, E.VRUNITARIOTI, E.VRESTOQTI
                               FROM (SELECT E.CDEMPRESATI, E.CDFILIALTI, E.NRNUMERORETI, E.DTCONTABILTI, E.CDFILIAL2TI,
                                            E.INSCRICAOTI, GETPRODUTO(E.PRODUTOTI, P_IDFUNCAOFILI) PRODUTOTI,
                                            E.QUANTIDADETI * GETFATORCONV(E.PRODUTOTI, P_IDFUNCAOFILI) QUANTIDADETI,
                                            E.VRUNITARIOTI, E.VRESTOQTI
                                       FROM EXPORTACAOTI E
                                      WHERE E.CDFILIAL2TI  = P_FILIAL
                                        AND E.DTCONTABILTI BETWEEN TO_CHAR(TO_DATE(P_DATAINI), 'YYYYMMDD') AND TO_CHAR(TO_DATE(P_DATAFINAL), 'YYYYMMDD')) E
                               LEFT OUTER JOIN USR_SAP_EXPORTACAOTI U
                                            ON E.CDEMPRESATI  = U.CDEMPRESATI
                                           AND E.CDFILIALTI   = U.CDFILIALTI
                                           AND E.NRNUMERORETI = U.NRNUMERORETI
                                           AND E.CDFILIAL2TI  = U.CDFILIAL2TI
                                           AND E.INSCRICAOTI  = U.INSCRICAOTI
                                           AND E.QUANTIDADETI = U.QUANTIDADETI
                                           AND E.VRUNITARIOTI = U.VRUNITARIOTI
                                           AND E.VRESTOQTI    = U.VRESTOQTI
                                           AND E.PRODUTOTI    = U.PRODUTOTI
                              WHERE U.CDEMPRESATI  IS NULL
                                AND U.CDFILIALTI   IS NULL
                                AND U.NRNUMERORETI IS NULL
                                AND U.CDFILIAL2TI  IS NULL
                                AND U.INSCRICAOTI  IS NULL);

   BEGIN
      /* Cursor da ExportacaoTI */
      OPEN C_EXPORTATI;
      LOOP
         /* Executa um loop com os registros que podem ser exportados */
         FETCH C_EXPORTATI INTO V_CDEMPRESATI, V_CDFILIALTI, V_NRNUMERORETI, V_DTCONTABILTI, V_CDFILIAL2TI,
                                V_INSCRICAOTI, V_PRODUTOTI, V_QUANTIDADETI, V_VRUNITARIOTI, V_VRESTOQTI;

         EXIT WHEN C_EXPORTATI%NOTFOUND;

         TI_SAP.CDEMPRESATI  := V_CDEMPRESATI;
         TI_SAP.CDFILIALTI   := V_CDFILIALTI;
         TI_SAP.NRNUMERORETI := V_NRNUMERORETI;
         TI_SAP.DTCONTABILTI := V_DTCONTABILTI;
         TI_SAP.CDFILIAL2TI  := V_CDFILIAL2TI;
         TI_SAP.INSCRICAOTI  := V_INSCRICAOTI;
         TI_SAP.PRODUTOTI    := V_PRODUTOTI;
         TI_SAP.QUANTIDADETI := V_QUANTIDADETI;
         TI_SAP.VRUNITARIOTI := V_VRUNITARIOTI;
         TI_SAP.VRESTOQTI    := V_VRESTOQTI;

         /* Cada linha que pode ser exportada eh gravada em TI_SAP, e a funcao retorna
            uma tabela para que o procedimento EXPORTA_TI_SAP seja executado */
         PIPE ROW (TI_SAP);
      END LOOP;

      RETURN;

   END GET_TI;

   PROCEDURE EXPORTA_TI_SAP(P_FILIAL    IN VARCHAR2,
                            P_DATAINI   IN DATE,
                            P_DATAFINAL IN DATE,
                            P_TCDFILIAL IN VARCHAR2)
   IS
      V_CDARVPRODORI VARCHAR2(20);
      V_IDFUNCAOFILI VARCHAR2(1);
      V_VRFATOCONV FLOAT;
      V_QUANTIDADE FLOAT;
      V_VRUNITARIO FLOAT;
      V_FILIAL     VARCHAR2(4);
      /* Retorna a filial e a funcao da mesma para a saber se os valores sao de OL ou nao */
      CURSOR C_FILIAL IS SELECT CDFILIAL, NVL(IDFUNCAOFILI, 'F')
                           FROM FILIAL
                          WHERE ((INSTR(P_FILIAL, '_' || CDFILIAL || '_') > 0) OR (P_TCDFILIAL = 'T'))
                          ORDER BY CDFILIAL;
   BEGIN
      /* Cursor da filial */
      OPEN C_FILIAL;
      LOOP
         /* Grava as variaveis de filial e funcao da filial */
         FETCH C_FILIAL INTO V_FILIAL, V_IDFUNCAOFILI;
         EXIT WHEN C_FILIAL%NOTFOUND; -- Termina a execucao se nao houver mais dados.

         /* Funcao que retorna os dados a serem exportados */
         FOR C_EXPORTACAOTI IN (SELECT * FROM TABLE(GET_TI(V_FILIAL, P_DATAINI, P_DATAFINAL, V_IDFUNCAOFILI)))
         LOOP
            V_CDARVPRODORI := C_EXPORTACAOTI.PRODUTOTI;
            V_QUANTIDADE   := C_EXPORTACAOTI.QUANTIDADETI;
            V_VRUNITARIO   := C_EXPORTACAOTI.VRUNITARIOTI;

            INSERT INTO USR_SAP_EXPORTACAOTI(CDEMPRESATI, CDFILIALTI, NRNUMERORETI, DTCONTABILTI, CDFILIAL2TI, FIXO2TI,
                                             INSCRICAOTI, PRODUTOTI, QUANTIDADETI, VRUNITARIOTI, VRESTOQTI,
                                             IDINTERFACE, TPINTERFACE, DTINTERFACEORI, DTINTERFACEDEST, STATUS, APPORIGEM, APPDESTINO)
            VALUES (C_EXPORTACAOTI.CDEMPRESATI, C_EXPORTACAOTI.CDFILIALTI, C_EXPORTACAOTI.NRNUMERORETI, C_EXPORTACAOTI.DTCONTABILTI, C_EXPORTACAOTI.CDFILIAL2TI, 'J',
                    C_EXPORTACAOTI.INSCRICAOTI, V_CDARVPRODORI, V_QUANTIDADE, V_VRUNITARIO, C_EXPORTACAOTI.VRESTOQTI,
                    USR_SAP_IDINTERFACE.NEXTVAL, 'I', SYSDATE, NULL, 1, 1, 2);

         END LOOP; -- FINALIZA LOOP DA EXPORTACAO TI
         -- COMMIT;
      END LOOP; -- FINALIZA LOOP DAS FILIAIS

      COMMIT; -- Commit executado ao final do processo para garantir que todas informacoes estejam exportadas
   END EXPORTA_TI_SAP;

END INT_SAP_TEK_TI;
/

create or replace
PROCEDURE EXPORTA_TI_SAP(P_FILIAL    IN VARCHAR2,
                         P_DATAINI   IN DATE,
                         P_DATAFINAL IN DATE,
                         P_TCDFILIAL IN VARCHAR2)
IS

BEGIN
   INT_SAP_TEK_TI.EXPORTA_TI_SAP(P_FILIAL, P_DATAINI, P_DATAFINAL, P_TCDFILIAL); 
END;
/
-- VERSAO --
BEGIN ATUVEROBJ('EXPORTA_TI_SAP','5.72.XXX'); END;
/